@@ -11,6 +11,7 @@ import getpass
 import platform
 import math
 
+
 class Vector3:
 
     """Basic encapsulation of of a 3D vector with several utility methods.
@@ -37,7 +38,7 @@ class Vector3:
     @classmethod
     def from_dict(cls, vector_dict):
 
-        """Instantiates ``Vector3`` from dictionary of components
+        """Instantiates ``Vector3`` from dictionary of components.
 
         Parameters
         ----------
@@ -51,9 +52,37 @@ class Vector3:
 
         return cls(vector_dict["x"], vector_dict["y"], vector_dict["z"])
 
+    @classmethod
+    def zero(cls):
+
+        """Instantiates a zero vector.
+
+        Returns
+        -------
+        Vector3
+            The zero vector
+        """
+
+        return Vector3(0, 0, 0)
+
     def __init__(self, x, y, z):
         self.x, self.y, self.z = x, y, z
 
+    def __neg__(self):
+        return Vector3(-self.x, -self.y, -self.z)
+
+    def __add__(self, other):
+        return Vector3(self.x + other.x, self.y + other.y, self.z + other.z)
+
+    def __sub__(self, other):
+        return Vector3(self.x - other.x, self.y - other.y, self.z - other.z)
+
+    def __mul__(self, other):
+        return Vector3(self.x * other, self.y * other, self.z * other)
+
+    def __truediv__(self, other):
+        return self * (1.0 / other)
+
     def magnitude(self):
 
         """Gets the magnitude of the vector.
@@ -75,7 +104,7 @@ class Vector3:
             The normalized vector
         """
         magnitude = self.magnitude()
-        return Vector3(self.x / magnitude, self.y / magnitude, self.z / magnitude)
+        return self / magnitude
 
 
 class Sensor:
@@ -97,9 +126,10 @@ class Altimeter(Sensor):
         The robot's altitude above the terrain
     """
 
-    def __init__(self, info_dict):
+    def __init__(self, info_object_reference, info_dict):
 
         self.altitude = None
+        self.__info_object_reference = info_object_reference
         super().__init__(info_dict)
 
     def update(self, info_dict):
@@ -117,9 +147,10 @@ class GPS(Sensor):
         The robot's world position
     """
 
-    def __init__(self, info_dict):
+    def __init__(self, info_object_reference, info_dict):
 
         self.position = None
+        self.__info_object_reference = info_object_reference
         super().__init__(info_dict)
 
     def update(self, info_dict):
@@ -147,11 +178,13 @@ class Gyroscope(Sensor):
         The normalized rightwards vector of the robot
     """
 
-    def __init__(self, info_dict):
+    def __init__(self, info_object_reference, info_dict):
 
         self.forward = None
         self.up = None
         self.right = None
+        self.is_upside_down = False
+        self.__info_object_reference = info_object_reference
         super().__init__(info_dict)
 
     def update(self, info_dict):
@@ -159,8 +192,14 @@ class Gyroscope(Sensor):
         gyroscope_dict = info_dict["gyroscope"]
 
         self.forward = Vector3.from_dict(gyroscope_dict["forward"])
-        self.up = Vector3.from_dict(gyroscope_dict["up"])
-        self.right = Vector3.from_dict(gyroscope_dict["right"])
+
+        self.up = Vector3.from_dict(gyroscope_dict["up"]) \
+            * self.__info_object_reference.coordinates_are_inverted
+
+        self.right = Vector3.from_dict(gyroscope_dict["right"]) \
+            * self.__info_object_reference.coordinates_are_inverted
+
+        self.is_upside_down = gyroscope_dict["isUpsideDown"]
 
 
 class Radar(Sensor):
@@ -176,10 +215,11 @@ class Radar(Sensor):
         Ping of "it" in classic tag. Evaluates to the zero vector if the robot is not playing classic tag.
     """
 
-    def __init__(self, info_dict):
+    def __init__(self, info_object_reference, info_dict):
 
         self.pings = None
         self.it_ping = None
+        self.__info_object_reference = info_object_reference
         super().__init__(info_dict)
 
     def update(self, info_dict):
@@ -210,9 +250,10 @@ class LiDAR(Sensor):
         ``flip_coordinates()``, left becomes right and up becomes down, just like everything else.
     """
 
-    def __init__(self, info_dict):
+    def __init__(self, info_object_reference, info_dict):
 
         self.distance_matrix = None
+        self.__info_object_reference = info_object_reference
         super().__init__(info_dict)
 
     def update(self, info_dict):
@@ -223,9 +264,18 @@ class LiDAR(Sensor):
         each have a field "array" containing a row."""
 
         self.distance_matrix = []
-        for container in container_array:
 
-            self.distance_matrix.append(container["array"])
+        if self.__info_object_reference.coordinates_are_inverted:
+
+            # flips up-down and left-right
+            for container in container_array[::-1]:
+
+                self.distance_matrix.append(container_array["array"][::-1])
+        else:
+
+            for container in container_array:
+
+                self.distance_matrix.append(container["array"])
 
 
 class RobotInfo:
@@ -260,11 +310,13 @@ class RobotInfo:
 
     def __init__(self, info_dict):
 
-        self.altimeter = Altimeter(info_dict)
-        self.gps = GPS(info_dict)
-        self.gyroscope = Gyroscope(info_dict)
-        self.lidar = LiDAR(info_dict)
-        self.radar = Radar(info_dict)
+        self.coordinates_are_inverted = False
+
+        self.altimeter = Altimeter(self, info_dict)
+        self.gps = GPS(self, info_dict)
+        self.gyroscope = Gyroscope(self, info_dict)
+        self.lidar = LiDAR(self, info_dict)
+        self.radar = Radar(self, info_dict)
 
         self.timestamp = info_dict["timestamp"]
         self.isIt = info_dict["isIt"]
@@ -304,7 +356,7 @@ class RobotConnection:
         self.__queue_path = data_dir + "EventQueue"
         self.__info_path = data_dir + "RobotState.json"
         self.__event_buffer = []  # where events that need to be sent are stored
-        self.info = None  # type: RobotInfo
+        self.info = None
         self.__should_destroy = False  # should this connection end
         self.__event_buffer_lock = threading.Lock()  # lock for threads wanting to access event_buffer
         self.__info_lock = threading.Lock()
@@ -328,7 +380,7 @@ class RobotConnection:
         None
         """
 
-        self.__queue_event("COORDINATES flip")
+        self.info.coordinates_are_inverted = True
 
     def set_tire_torque(self, tire_name, torque):
 
@@ -346,6 +398,9 @@ class RobotConnection:
         None
         """
 
+        if self.info.coordinates_are_inverted:
+            torque *= -1
+
         self.__queue_event("SET tire " + tire_name + " " + str(torque))
 
     def set_tire_steering(self, tire_name, bearing):
@@ -363,6 +418,8 @@ class RobotConnection:
         -------
         None
         """
+        if self.info.coordinates_are_inverted:
+            bearing += -1
 
         self.__queue_event("SET steering " + tire_name + " " + str(bearing))
 
@@ -418,7 +475,7 @@ class RobotConnection:
                     else:
                         self.info = RobotInfo(info_dict)
 
-            except (EnvironmentError, json.JSONDecodeError) as e:
+            except (EnvironmentError, json.JSONDecodeError):
                 continue
 
             state_file.close()
@@ -426,7 +483,7 @@ class RobotConnection:
 
     def connect(self):
 
-        """Starts the connection to AutonoWar
+        """Starts the connection to AutonoWar.
 
         Returns
         -------
